@@ -14,6 +14,7 @@
 // do anything - all the magic is handled entirely by
 // autocxx_macro::include_cpp_impl.
 
+mod rvalue_param;
 pub mod subclass;
 mod value_param;
 
@@ -274,6 +275,8 @@ macro_rules! exclude_impls {
 
 /// Indicates that a C++ type is not to be generated by autocxx in this case,
 /// but instead should refer to some pre-existing Rust type.
+/// Note that the size and alignment of this type *must* be correct.
+/// If you wish for the type to be POD, you can use a `pod!` directive too.
 ///
 /// The syntax is:
 /// `extern_cpp_type!("CppNameGoesHere", path::to::rust::type)`
@@ -285,6 +288,21 @@ macro_rules! extern_cpp_type {
     ($($tt:tt)*) => { $crate::usage!{$($tt)*} };
 }
 
+/// Indicates that a C++ type is not to be generated by autocxx in this case,
+/// but instead should refer to some pre-existing Rust type. Unlike
+/// `extern_cpp_type!`, there's no need for the size and alignment of this
+/// type to be correct.
+///
+/// The syntax is:
+/// `extern_cpp_opaque_type!("CppNameGoesHere", path::to::rust::type)`
+///
+/// A directive to be included inside
+/// [include_cpp] - see [include_cpp] for general information.
+#[macro_export]
+macro_rules! extern_cpp_opaque_type {
+    ($($tt:tt)*) => { $crate::usage!{$($tt)*} };
+}
+
 /// Deprecated - use [`extern_rust_type`] instead.
 #[macro_export]
 #[deprecated]
@@ -478,6 +496,8 @@ pub trait WithinBox {
     fn within_box(self) -> Pin<Box<Self::Inner>>;
 }
 
+use cxx::kind::Trivial;
+use cxx::ExternType;
 use moveit::MakeCppStorage;
 use moveit::{Emplace, EmplaceUnpinned};
 
@@ -502,9 +522,51 @@ where
     }
 }
 
+/// Emulates the [`WithinUniquePtr`] trait, but for trivial (plain old data) types.
+/// This allows such types to behave identically if a type is changed from
+/// `generate!` to `generate_pod!`.
+///
+/// (Ideally, this would be the exact same trait as [`WithinUniquePtr`] but this runs
+/// the risk of conflicting implementations. Negative trait bounds would solve
+/// this!)
+pub trait WithinUniquePtrTrivial: UniquePtrTarget + Sized + Unpin {
+    fn within_unique_ptr(self) -> cxx::UniquePtr<Self>;
+}
+
+impl<T> WithinUniquePtrTrivial for T
+where
+    T: UniquePtrTarget + ExternType<Kind = Trivial> + Sized + Unpin,
+{
+    fn within_unique_ptr(self) -> cxx::UniquePtr<T> {
+        UniquePtr::new(self)
+    }
+}
+
+/// Emulates the [`WithinBox`] trait, but for trivial (plain old data) types.
+/// This allows such types to behave identically if a type is changed from
+/// `generate!` to `generate_pod!`.
+///
+/// (Ideally, this would be the exact same trait as [`WithinBox`] but this runs
+/// the risk of conflicting implementations. Negative trait bounds would solve
+/// this!)
+pub trait WithinBoxTrivial: Sized + Unpin {
+    fn within_box(self) -> Pin<Box<Self>>;
+}
+
+impl<T> WithinBoxTrivial for T
+where
+    T: ExternType<Kind = Trivial> + Sized + Unpin,
+{
+    fn within_box(self) -> Pin<Box<T>> {
+        Pin::new(Box::new(self))
+    }
+}
+
 use cxx::memory::UniquePtrTarget;
 use cxx::UniquePtr;
 use moveit::New;
+pub use rvalue_param::RValueParam;
+pub use rvalue_param::RValueParamHandler;
 pub use value_param::as_copy;
 pub use value_param::as_mov;
 pub use value_param::as_new;
@@ -529,9 +591,12 @@ pub mod prelude {
     pub use crate::cpp_semantics;
     pub use crate::include_cpp;
     pub use crate::PinMut;
+    pub use crate::RValueParam;
     pub use crate::ValueParam;
     pub use crate::WithinBox;
+    pub use crate::WithinBoxTrivial;
     pub use crate::WithinUniquePtr;
+    pub use crate::WithinUniquePtrTrivial;
     pub use cxx::UniquePtr;
     pub use moveit::moveit;
     pub use moveit::new::New;
